/*
 * Copyright (c) 2016-2020 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#define LOG_TAG "BnHiccupd"

#include "BnHiccupd.hpp"
#include <cstdio>
#include <cstdlib>
#include <memory>

#include <sys/stat.h>
#include <fcntl.h>
#include <binder/IPCThreadState.h>
#include <log/log.h>

using namespace android;

#define PLOG aout

namespace hiccup {

namespace {

status_t sendLogFiles(const Vector<LogFile> &v, Parcel &parcel) {
  status_t ret = OK;
  if ( (ret = parcel.writeInt32(v.size())) != OK) {
    return ret;
  }
  for (const auto& logFile : v) {
    String8 pathLeaf = String8(logFile.fileName).getPathLeaf();
    ALOGD("sending fd for %s ...", pathLeaf.string());

    if ( (ret = parcel.writeString16(String16(pathLeaf))) != OK) {
      return ret;
    }
    ret = parcel.writeFileDescriptor(logFile.fd, true); /* binder will close our fds */
    if (ret != OK) {
      ALOGD("Writing Filedescriptor failed: %d", ret);
      return ret;
    }
  }
  return ret;
}

}

status_t BnHiccupd::onTransact(uint32_t code, const Parcel& data, Parcel* reply, uint32_t flags) {
  ALOGD("%s(%i) %i", __func__, code, flags);
  data.checkInterface(this);
  data.print(PLOG); endl(PLOG);
  status_t ret = BAD_VALUE;
  String16 bootreason;
  switch(code) {
    case GET_LAST_KMESG:
      if (auto v = std::unique_ptr<Vector<LogFile>>(getLastKmsg())) {
        ret = sendLogFiles(*v, *reply);
      }
      return ret;
    case GET_BOOTREASON:
      bootreason= getBootReason();
      ret = reply->writeString16(bootreason);
      return ret;
    case GET_POWERONREASON:
      bootreason= getPowerOnReason();
      ret = reply->writeString16(bootreason);
      return ret;
    case GET_POWEROFFREASON:
      bootreason= getPowerOffReason();
      ret = reply->writeString16(bootreason);
      return ret;
    case GET_API_VERSION:
      ret = reply->writeInt32(getApiVersion());
      return ret;
    default:
      return BBinder::onTransact(code, data, reply, flags);
  }
}

}  /* namespace hiccup */
